// THINDUMP.CPP
//
// Copyright (c) 1999-2001 Symbian Ltd. All rights reserved.

#include <e32test.h>
#include <f32file.h>
#include <s32file.h>
#include <c32comm.h>
#include <bacline.h>

GLDEF_D RTest gTest(_L("thindump utility"));
GLDEF_D RFs gFs;

#if defined (__WINS__)
#define PDD_NAME _L("ECDRV")
#define LDD_NAME _L("ECOMM")
#else
#define PDD_NAME _L("EUART1")
#define LDD_NAME _L("ECOMM")
#endif

LOCAL_C void DumpFile()
//
// Dump narrow text file to console, comms & debug (WINS)
//
	{

	TInt ret;
	_LIT(KDumpFileDefault, "c:\\log.txt");

	// Obtain command line parameters
	TPtrC filename( KDumpFileDefault );
	CCommandLineArguments* args = CCommandLineArguments::NewLC();
	if (args->Count() > 1)
		filename.Set(args->Arg(1));

	// Open file and determine size
	RFile file;
	ret = file.Open(gFs, filename, EFileStreamText | EFileShareAny);
	User::LeaveIfError(ret);

	CleanupStack::PopAndDestroy( /* args */ );

	TInt size;
	ret = file.Size(size);
	User::LeaveIfError(ret);
//	gTest.Printf(_L("%d bytes\n"), size);

	// Get ready to read file
#define BUF_SIZE 32
#define MAX_LINE_LENGTH 1024

	_LIT(KCharLF, "\x0a");
	_LIT(KCharCR, "\x0d");

	TInt pos = 0;
	TInt lfPos;
	TInt crPos;
	TInt lineCount = 0;
	TInt consoleHeight = gTest.Console()->ScreenSize().iHeight - 5;
	TInt consoleWidth = gTest.Console()->ScreenSize().iWidth;
	TInt linesToScroll = consoleHeight;

	TBuf8<BUF_SIZE+1> buf8(BUF_SIZE+1);
	TBuf<BUF_SIZE+1> buf16(BUF_SIZE+1);

	HBufC* format16=HBufC::NewLC(MAX_LINE_LENGTH);

	// Read block from file
	while(pos < size)
		{
		ret = file.Read(buf8, BUF_SIZE);
		User::LeaveIfError(ret);

		// Expand to 16 bit chars
		buf16.Copy(buf8);

		// Remove line feeds
		while ( (crPos = buf16.Find(KCharCR)) != KErrNotFound )
			buf16.Delete(crPos, 1);

		// Find carriage returns
		TPtr ptr16( format16->Des() );
		while ( (lfPos = buf16.Find(KCharLF)) != KErrNotFound )
			{
			// Extract this line & append to any buffered line
			TPtrC ptrLeft = buf16.Left(lfPos+1);
			ptr16.Append(ptrLeft);
			buf16.Delete(0, lfPos+1);

			// Update line count displayed, pausing if required
			lineCount += (ptr16.Length() / consoleWidth) + 1;
			if (lineCount >= linesToScroll)
				{
				ret = gTest.Getch();
				if (ret == 'q')
					User::Leave(0);
				else if (ret == '\r')
					linesToScroll = 1;
				else
					linesToScroll = consoleHeight;
				lineCount = 0;
				}

			// Print this line
			gTest.Printf(_L("%S"), &ptr16 );
			ptr16.Zero();
			}

		ptr16.Append(buf16);
		pos += BUF_SIZE;
		}

	// Output any partial line still in buffer
	if (format16->Length())
		gTest.Printf(_L("%S\r\n"), format16);

	CleanupStack::PopAndDestroy();

	file.Close();
	}


LOCAL_C TInt InitGlobals()
//
// Initialise global variables.
//
	{

	TInt err;

	err=User::LoadPhysicalDevice(PDD_NAME);
	if (err!=KErrNone && err!=KErrAlreadyExists)
		return(err);
	err=User::LoadLogicalDevice(LDD_NAME );
	if (err!=KErrNone && err!=KErrAlreadyExists)
		return(err);
//	StartC32();
	
	err=gFs.Connect();
	gTest(err==KErrNone);

	// Force console creation
	gTest.Printf(_L(""));

	return KErrNone;
	}

LOCAL_C void DestroyGlobals()
//
// Free global variables
//
	{
	gFs.Close();
	}

LOCAL_C void RunDumpL()
//
// Run all the tests
//
	{

	TInt ret = InitGlobals();
	if(ret != KErrNone)
		return;

	DumpFile();
	DestroyGlobals();
	}

EXPORT_C TInt E32Main()
//
// Main
//
	{

	CTrapCleanup* cleanup = CTrapCleanup::New();
	CActiveScheduler* theActiveScheduler = new CActiveScheduler();
	CActiveScheduler::Install(theActiveScheduler);

	__UHEAP_MARK;

//	gTest.Title();
//	Test.Start(_L(""));

	TRAPD(err,RunDumpL());
	if (err!=KErrNone)
		gTest.Printf(_L("ERROR: Leave %d\n"),err);

	gTest.Printf(_L("Dump complete. Press any key ...\n"));
	gTest.Getch();
//	gTest.End();
	gTest.Close();

	__UHEAP_MARKEND;

	delete cleanup;
	delete theActiveScheduler;
	return KErrNone;
	}